@@ -1,10 +1,11 @@
 # -*- coding: utf-8 -*-
 from __future__ import print_function
 
-import warnings
-
-from clu.constants.consts import BUILTINS, DEBUG, QUALIFIER, NoDefault
-from clu.exporting import determine_name, path_to_dotpath, Exporter
+from clu.constants.consts import BASEPATH, DEBUG, QUALIFIER, NoDefault
+from clu.exporting import (path_to_dotpath, determine_name,
+                                            search_for_name,
+                                            search_for_module)
+from clu.exporting import Exporter, Registry
 
 exporter = Exporter(path=__file__)
 export = exporter.decorator()
@@ -14,39 +15,18 @@ export = exporter.decorator()
 # by thing names or id(thing) values
 
 @export
-def itermodule(module):
-    """ Get an iterable of `(name, thing)` tuples for all things
-        contained in a given module (although it’ll probably work
-        for classes and instances too – anything `dir()`-able.)
+def determine_module(thing, name=None):
+    """ Private module function to find the module of a thing,
+        using “pickle.whichmodule(…)”
     """
-    keys = tuple(key for key in sorted(dir(module)) \
-                      if key not in BUILTINS)
-    values = (getattr(module, key) for key in keys)
-    return zip(keys, values)
-
-@export
-def moduleids(module):
-    """ Get a dictionary of `(name, thing)` tuples from a module,
-        indexed by the `id()` value of `thing`
-    """
-    out = {}
-    for key, thing in itermodule(module):
-        out[id(thing)] = (key, thing)
-    return out
-
-@export
-def thingname(original, *modules):
-    """ Find the name of a thing, according to what it is called
-        in the context of a module in which it resides
-    """
-    inquestion = id(original)
-    with warnings.catch_warnings():
-        warnings.simplefilter("ignore")
-        for module in frozenset(modules):
-            for key, thing in itermodule(module):
-                if id(thing) == inquestion:
-                    return key
-    return None
+    import pickle
+    
+    if name is not None:
+        return name
+    if thing is None:
+        return None
+    
+    return pickle.whichmodule(thing, None)
 
 @export
 def nameof(thing, default=NoDefault):
@@ -56,22 +36,32 @@ def nameof(thing, default=NoDefault):
         >>> thing.__name__
         … optionally specifying a “default” fallback.
     """
+    from clu.predicates import pyname
+    
+    result = pyname(thing) or \
+             Registry.nameof(thing) or \
+             search_for_name(thing)
+    
     if default is NoDefault:
-        return determine_name(thing)
-    return determine_name(thing) or default
+        return dotpath_split(result)[0]
+    return dotpath_split(result)[0] or default
 
 @export
-def determine_module(thing):
+def moduleof(thing, default=NoDefault):
     """ Determine in which module a given thing is ensconced,
         and return that modules’ name as a string.
     """
-    import pickle
-    from clu.exporting import thingname_search_by_id
-    from clu.predicates import pyattr
-    return pyattr(thing, 'module', 'package') or \
-           determine_name(
-           thingname_search_by_id(id(thing))[0]) or \
-           pickle.whichmodule(thing, None)
+    from clu.predicates import pymodule
+    
+    result = pymodule(thing) or \
+             determine_name(
+                 Registry.moduleof(thing) or \
+                 search_for_module(thing)) or \
+             determine_module(thing)
+    
+    if default is NoDefault:
+        return result
+    return result or default
 
 # QUALIFIED-NAME FUNCTIONS: import by qualified name (like e.g. “yo.dogg.DoggListener”),
 # assess a thing’s qualified name, etc etc.
@@ -82,18 +72,21 @@ def dotpath_join(base, *addenda):
     if base is None or base == '':
         return dotpath_join(*addenda)
     for addendum in addenda:
-        if not base.endswith(QUALIFIER):
-            base += QUALIFIER
-        if addendum.startswith(QUALIFIER):
-            if len(addendum) == 1:
-                raise ValueError(f'operand too short: {addendum}')
-            addendum = addendum[1:]
-        base += addendum
+        if addendum is not None:
+            if not base.endswith(QUALIFIER):
+                base += QUALIFIER
+            if addendum.startswith(QUALIFIER):
+                if len(addendum) == 1:
+                    raise ValueError(f'operand too short: {addendum}')
+                addendum = addendum[1:]
+            base += addendum
     # N.B. this might be overthinking it -- 
     # maybe we *want* to allow dotpaths
     # that happen to start and/or end with dots?
-    if base.endswith(QUALIFIER):
-        return base[:-1]
+    while base.startswith(QUALIFIER):
+        base = base[1:]
+    while base.endswith(QUALIFIER):
+        base = base[:-1]
     return base
 
 @export
@@ -103,6 +96,8 @@ def dotpath_split(dotpath):
         When called with a string containing no dots,
         `dotpath_split(…)` returns `(string, None)`.
     """
+    if dotpath is None:
+        return None, None
     head = dotpath.split(QUALIFIER)[-1]
     tail = dotpath.replace(f"{QUALIFIER}{head}", '')
     return head, tail != head and tail or None
@@ -127,9 +122,7 @@ def qualified_name_tuple(thing):
     """ Get the module/package and thing-name for a class or module.
         e.g. ('instakit.processors.halftone', 'FloydSteinberg')
     """
-    return determine_module(thing), \
-           dotpath_split(
-           determine_name(thing))[0]
+    return moduleof(thing), nameof(thing)
 
 @export
 def qualified_name(thing):
@@ -156,9 +149,10 @@ def dotpath_to_prefix(dotpath, sep='-', end='-'):
     return dotpath.lower().replace(QUALIFIER, sep) + end
 
 @export
-def path_to_prefix(path, sep='-', end='-'):
+def path_to_prefix(path, sep='-', end='-', relative_to=BASEPATH):
     """ Shortcut for dotpath_to_prefix(path_to_dotpath(…)) """
-    return dotpath_to_prefix(path_to_dotpath(path),
+    return dotpath_to_prefix(path_to_dotpath(path,
+                                             relative_to=relative_to),
                              sep=sep,
                              end=end)
 
